/*
 * c99 complex mathematical functions
 *
 * 2016-08 Kouichi Hirabayashi
 */

#include <stdio.h>
#include <stdlib.h>
#include "Complex.h"

COMPLEX Sqrt(x) COMPLEX x;
{
  double c, r, w;
  COMPLEX z;

  c = sqrt(0.5);
  r = cabs(x);
  w = sqrt(r + fabs(creal(x)));
  if (w == 0.0)
	z = (COMPLEX)0.0;
  else if (creal(x) >= 0)
	z = c * w + I * c * cimag(x) / w;
  else
	z = c * fabs(cimag(x)) / w
		+ I * ((cimag(x) >= 0.0) ? c * w : -c * w);
  return z;
}

COMPLEX Exp(x) COMPLEX x;
{
  double a;

  a = exp(creal(x));
  x = a * cos(cimag(x)) + a * sin(cimag(x)) * I;
  return x;
}

COMPLEX Log(x) COMPLEX x;
{
  COMPLEX z;

  z = 0.5 * log(creal(x) * creal(x) + cimag(x) * cimag(x))
	+  atan2(cimag(x), creal(x)) * I;
  return z;
}

COMPLEX Pow(x, y) COMPLEX x, y;
{
  if (x == (COMPLEX)0.0)
	return (COMPLEX)0.0;
  if (y == (COMPLEX)0.0)
	return (COMPLEX)1.0;
  if (y == (COMPLEX)1.0)
	return x;
  return Exp(y* Log(x));
}

// Sin(x) = sin(x.x)*cosh(x.y) + j*cos(x.x)*sinh(x.y)

COMPLEX Sin(x) COMPLEX x;
{
  double e, f;

  e = exp(cimag(x)); f = 1 / e;
  x = 0.5 * cos(creal(x)) * (e - f) + 0.5 * sin(creal(x)) * (e + f) * I;
  return x;
}

// Cos(x) = cos(x.x)*cosh(x.y) + j*sin(x.x)*sinh(x.y)

COMPLEX Cos(x) COMPLEX x;
{
  double e, f;

  e = exp(cimag(x)); f = 1.0 / e;
  x = 0.5 * cos(creal(x)) * (f + e) + 0.5 * sin(creal(x)) * (f - e) * I;
  return x;
}

COMPLEX Tan(x) COMPLEX x;
{
  double e, f, d;

  e = exp(2.0 * cimag(x)); f = 1.0 / e;
  d = cos(2.0 * creal(x)) + 0.5 * (e + f);
  x = sin(2.0 * creal(x)) / d + 0.5 * (e - f) / d * I;
  return x;
}

COMPLEX Sinh(x) COMPLEX x;
{
  double e, f;

  e = exp(creal(x)); f = 1.0 / e;
  x = 0.5 * (e - f) * cos(cimag(x)) + 0.5 * (e + f) * sin(cimag(x)) * I;
  return x;
}

COMPLEX Cosh(x) COMPLEX x;
{
  double e, f;

  e = exp(creal(x)); f = 1.0 / e;
  x = 0.5 * (e + f) * cos(cimag(x)) + 0.5 * (e - f) * sin(cimag(x)) * I;
  return x;
}

COMPLEX Tanh(x) COMPLEX x;
{
  double e, f, d;

//  return Sinh(x) / Cosh(x);
  e = exp(2.0 * creal(x)); f = 1.0 / e;
  d = 0.5 * (e + f) + cos(2.0 * cimag(x));
  x = 0.5 * (e - f) / d + sin(2.0 * cimag(x)) / d;
  return x;

}

COMPLEX Coth(x) COMPLEX x;
{
  return Cosh(x) / Sinh(x);
}

COMPLEX Atanh(x) COMPLEX x;
{
  return Log((1.0 + x) / (1.0 - x)) * 0.5;
}

COMPLEX Acoth(x) COMPLEX x;
{
  return Log((x + 1.0) / (x - 1.0)) * 0.5;
}

/*
 * Modefied Bessel Functions
 *
 * Cf. Handbook of Mathematical Functions
 *	With Formulas, Graphs, and Mathematical Tables
 *		(U.S. Department of Commerce, National Bureau of Standards)
 */

COMPLEX Jns(n, x) int n; COMPLEX x;
{
  int i;
  double g;
  COMPLEX N, D, S, X, A;

  if (x == 0.0)
	return (n == 0) ? (COMPLEX)1.0 : (COMPLEX)0.0;
  if (70.0 < cabs(x))
//  if (50.0 < cabs(x))
//  if (10.0 < cabs(x))
//  if (5.0 < cabs(x))
	return Jn(n, x);
  N = D = S = 1.0;
  X = x * x;
  for (i = 1; i <= 100; i++) {
	N *= X;
	D *= 4.0 * (double)(i * (n + i));
//	D *= (double)(2 * i * (2 * n + 2 * i));
	A = N / D;
	if ((i % 2) == 1)
		S -= A;
	else
		S += A;
	if (cabs(A) / cabs(S) < 1e-6)
		break;
  }
//if (i >= 500) fprintf(stderr, "! Jn() i(%d)\n", i);
#if 0
  g = 1.0;
  for (i = 2; i <= n; i++)
	g *= (double)i;
  X = x * 0.5;
  return S / g * Pow(X, (COMPLEX)n);
#else
  if (n == 0)
	return S;
  N = X = 0.5 * x;
  for (i = 2; i <= n; i++)
	N *= X / (COMPLEX)i;
  return S * N;
#endif
}

COMPLEX Ins(n, x) int n; COMPLEX x;
{
  int i;
  double g;
  COMPLEX N, D, S, X, A;

  N = D = S = 1.0;
  X = x * x;
  for (i = 1; i <= 100; i++) {
	N *= X;
	D *= 4.0 * (COMPLEX)(i * (n + i));
	A = N / D;
	S += A;
	if (cabs(A) / cabs(S) < 1e-6)
		break;
  }
  if (n == 0)
	return S;
  N = X = 0.5 * x;
  for (i = 2; i <= n; i++)
	N *= X / (COMPLEX)i;
  return S * N;
}

COMPLEX In(n, x) int n; COMPLEX x;
{
  int i;
  double g;
  COMPLEX z, z1, z2;

  if (n == 0)
	return I0(x);
  else if (n == 1)
	return I1(x);
  else if (cabs(x) < 1.0) {
	for (g = 1.0, i = 2; i <= n; i++)
		g *= (double)i;
	return Pow(x * 0.5, (double)n) / (double)g;
  }
  z1 = I0(x);
  z2 = I1(x);
  for (i = 1; i < n; i++) {
//printf("# n(%d), i(%d)\n", n, i);
	z = z1 - z2 * 2.0 * (double)i / x;
	z1 = z2;
	z2 = z;
  }
  return z;
}

COMPLEX Jn(n, x) int n; COMPLEX x;
{
  int i;
  double g;
  COMPLEX z, z1, z2;

  if (x == (COMPLEX)0.0)
	return (n == 0) ? (COMPLEX)1.0 : (COMPLEX)0.0;
  if (n == 0)
	return J0(x);
  else if (n == 1)
	return J1(x);
#if 1
  else if (cabs(x) < 1.0) {
	for (g = 1.0, i = 2; i <= n; i++)
		g *= (double)i;
	return Pow(x * 0.5, (COMPLEX)n) / g;
  }
#endif
  z1 = J0(x);
  z2 = J1(x);
  for (i = 1; i < n; i++) {
	z = z2 * 2.0 * (double)i /  x - z1;
	z1 = z2;
	z2 = z;
  }
  return z;
}

COMPLEX J0(x) COMPLEX x;
{	/* Bessel Function of the First order */
  static double a1[] = { .0002100, -.0039444, .0444479, -.3163866,
	1.2656208, -2.2499997 };
  static double a2[] = { .00014476, -.00072805, .00137237, -.00009512,
	-.00552740, -.00000077 };
  static double a3[] = { .00013558, -.00029333, -.00054125, .00262573,
	-.00003954, -.04166397 };

  int i;
  COMPLEX c, t, y, z;

  c = 3.0;
  if (cabs(x) <= cabs(c)) {
	t = x / c; t = t * t;
	y = 0.0;
	for (i = 0; i < 6; i++)
		y = (y + a1[i]) * t;
	y += 1.0;
  }
  else {
	t = c / x;
	y = 0.0;
	for (i = 0; i < 6; i++)
		y = (y + a2[i]) * t;
	y += 0.79788456;
	z = 0.0;
	for (i = 0; i < 6; i++)
		z = (z + a3[i]) * t;
	z -= .78539816;
	z += x;
	y = y * Cos(z) / csqrt(x);
  }
  return y;
}

COMPLEX J1(x) COMPLEX x;
{	/* Bessel Function of the Second order */
  static double a1[] = { .00001109, -.00031761, .00443319, -.03954289,
	.21093573, -.56249985 };
  static double a2[] = { -.00020033, .00113653, -.00249511, .00017105,
	.01659667, .00000156 };
  static double a3[] = { -.00029166, .00079824, .00074348, -.00637879,
	.00005650, .12499612 };
  int i;
  COMPLEX c, t, y, z;

  c = 3.0;
  if (cabs(x) <= cabs(c)) {
	t = x / c; t = t * t;
	y = 0.0;
	for (i = 0; i < 6; i++)
		y = (y + a1[i]) * t;
	y += 0.5;
	y *= x;
  }
  else {
	t = c / x;
	y = 0.0;
	for (i = 0; i < 6; i++)
		y = (y + a2[i]) * t;
	y += 0.79788456;
	z = 0.0;
	for (i = 0; i < 6; i++)
		z = (z + a3[i]) * t;
	z -= 2.35619449;
	z += x;
	y *= Cos(z) / csqrt(x);
  }
  return y;
}

COMPLEX I0(x) COMPLEX x;
{	/* Modified Bessel Function of the First order */
  static double a1[] = { 0.0045813, 0.0360768, 0.2659732, 1.2067492,
	3.0899424, 3.5156229 };
  static double a2[] = { 0.00392377, -0.01647633, 0.02635537, -0.02057706,
	0.00916281, -0.00157565, 0.00225319, 0.01328592 };

  int i;
  COMPLEX c, t, y;

  c = 3.75;
  if (cabs(x) <= cabs(c)) {
	t = x / c; t = t * t;
	y = 0.0;
	for (i = 0; i < 6; i++)
		y = (y + a1[i]) * t;
	y += 1.0;
  }
  else {
	t = c / x;
	y = 0.0;
	for (i = 0; i < 8; i++)
		y = (y + a2[i]) * t;
	y += 0.39894228;
	y *= Exp(x) / csqrt(x);
  }
  return y;
}


COMPLEX I1(x) COMPLEX x;
	{	/* Modified Bessel Function of the First order */
  static double a1[] = { 0.00032411, 0.00301532, 0.02658733, 0.15084934,
	0.51498869, 0.87890594 };
  static double a2[] = { -0.00420059, 0.01787654, -0.02895312, 0.02282967,
	-0.01031555, 0.00163801, -0.00362018, -0.03988024 };
  int i;
  COMPLEX c, t, y;

  c = 3.75;
  if (cabs(x) <= cabs(c)) {
	t = x / c; t = t * t;
	y = 0.0;
	for (i = 0; i < 6; i++)
		y = (y + a1[i]) * t;
	y += 0.5;
	y *= x;
  }
  else {
	t = c / x;
	y = 0.0;
	for (i = 0; i < 8; i++)
		y = (y + a2[i]) * t;
	y += 0.39894228;
	y *= Exp(x) / csqrt(x);
  }
  return y;
}

COMPLEX K0(x) COMPLEX x;
{	/* Modified Bessel Function of the Second order */
  static double a1[] = { 0.00000740, 0.00010750, 0.00262698, 0.03488590,
	0.23069756, 0.42278420 };
  static double a2[] = { 0.00053208, -0.00251540, 0.00587872, -0.01062446,
	0.02189568, -0.07832358 };
  int i;
  COMPLEX c, t, y;

  c = 2.0;
  if (cabs(x) <= cabs(c)) {
	t = x / c; t = t * t;

	y = 0.0;
	for (i = 0; i < 6; i++)
		y = (y + a1[i]) * t;
	y -= 0.57721566;
	y -=  Log(x * 0.5) * I0(x);
  }
  else {
	t = c / x;
	y = 0.0;
	for (i = 0; i < 6; i++)
		y = (y + a2[i]) * t;
	y += 1.25331414;
	y *= Exp(-x) / csqrt(x);
  }
  return y;
}

COMPLEX K1(x) COMPLEX x;
{	/* Modified Bessel Function of the Second order */
  int i;
  static double a1[] = { -0.00004686, -0.00110404, -0.01919402, -0.18156897,
	-0.67278579, 0.15443144 };
  static double a2[] = { -0.00068245, 0.00325614, -0.00780353, 0.01504268,
	-0.03655620, 0.23498619 };
  COMPLEX c, t, y;

  c = 2.0;
  if (cabs(x) <= cabs(c)) {
	t = x / c; t = t * t;
	y = 0.0;
	for (i = 0; i < 6; i++)
		y = (y + a1[i]) * t;
	y += 1.0;
	y = (y + x * Log(x * 0.5) * I1(x)) / x;
  }
  else {
	t = c / x;
	y = 0.0;
	for (i = 0; i < 6; i++)
		y = (y + a2[i]) * t;
	y += 1.25331414;
	y *= Exp(-x) / csqrt(x);
  }
  return y;
}
