/*
 * proximity effect correction factor by A.H.M.Arnold
 * 2016-09-25 Kouichi Hirabayashi
 *
 * Compiling: cc -o a a.c Complex.c -lm
 *	( a is the name of this program)
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

double P(double, double, double, double, double, double);

int main()
{
  double a, b, s, f, u, g, p, PI;

  PI = atan2(0.0, -1.0);
  a = 2.5e-4;		// conductor radius (m)
  b = 0.0;		// conductor innner radius for tube (m)
  s = 1.0e-3;		// conductor spacinge (m)
  u = 4e-7 * PI;	// permeability (H/m)
  g = 5.80e7;		// conductivitu (G/m)
  for (f = 1e2; f <= 1e9; f *= 1.01) {	// f = frequency (Hz)
	p = P(a, b, s, f, g, u);	// proximity effect correction factor
	printf("%g %g\n", f, p);
  }
  exit(0);
}

/*
 *  proximity effect correction factor with soloid or tubular conductor
 *
 *  P = Rac_with_proximity_effect_of_parallel_wire / Rac_with_skin_effect
 *	a = outer radius of the tubulat or solid conductor (m)
 *	b = inner radius of the tubular conductor (0 for solid conductor) (m)
 *	s = conductor spacing (m)
 *	f = angular frequency (Hz)
 *	g = conductivity (G/m)
 *	u = permiability (H/m)
 *
 *
 * This program is based on:
 * A.H.M.Arnold,- Proximity efeect in solid and hollow round conductors
 *	(Jour.I.E.E. vol 88, Part 2, pp349-359, 1941)
 */

static double _f1(double), _f2(double), _f3(double), _f4(double), _f5(double),
	_f6(double), _f7(double);

double P(a, b, s, f, g, u) double a, b, s, f, g, u;
{
  double at, d, t, A1, A2, A3, r, r2, s2, s4, x, P, PI;

  PI = atan2(0.0, -1.0);
  d = sqrt(2.0 / (2.0 * PI * f * u * g));	// skin depth
  t = a - b;	// thickness
  if (t < 0) {
	printf("a < b !\n");
	exit(1);
  }
  at = sqrt(2.0 * a * t - t * t);
  r = at / a;
  r2 = r * r;
  x = sqrt(2.0) * at / d;
  if (s < 2.0 * a) {
	printf("conductor spacing must be greater than conductor diameter !\n");
	exit(1);
  }
  A1 = _f1(x) + (1.0 - r - r2 * (1.0 - r) * _f7(x)) * _f4(x);
  A2 = _f2(x) + (1.0 - r2) * _f5(x);
  A3 = _f3(x) + (1.0 - r2) * _f6(x);
  s2 = 0.5 * s / a;
  s2 *= s2;
//  s4 = s2 * s2;
  P = 1.0 / sqrt(1.0 - A1 / s2 + A2 / s2 / (1.0 - A3 / s2));
  return P;
}

static double
_f7(x) double x;
{
  double w;

  w = x * x * x;
  return w / (400.0 + w);
}

static struct {
  double x, f1, f4;
} tab1[] = {
   0.0, 0.000, 0.000,
   0.2, 0.000, 0.000,
   0.3, 0.000, 0.000,
   0.4, 0.001, 0.001,
   0.5, 0.002, 0.002,
   0.6, 0.004, 0.004,
   0.7, 0.007, 0.007,
   0.8, 0.013, 0.012,
   0.9, 0.020, 0.019,
   1.0, 0.030, 0.029,
   1.1, 0.044, 0.040,
   1.2, 0.061, 0.054,
   1.3, 0.081, 0.070,
   1.4, 0.106, 0.088,
   1.5, 0.135, 0.106,
   1.6, 0.167, 0.123,
   1.7, 0.203, 0.140,
   1.8, 0.240, 0.156,
   1.9, 0.280, 0.169,
   2.0, 0.320, 0.180,
   2.1, 0.360, 0.189,
   2.2, 0.399, 0.196,
   2.3, 0.436, 0.201,
   2.4, 0.470, 0.205,
   2.5, 0.502, 0.208,
   2.6, 0.530, 0.210,
   2.7, 0.556, 0.213,
   2.8, 0.578, 0.215,
   2.9, 0.598, 0.218,
   3.0, 0.614, 0.221,
   3.1, 0.629, 0.224,
   3.2, 0.641, 0.227,
   3.3, 0.652, 0.230,
   3.4, 0.661, 0.233,
   3.5, 0.668, 0.235,
   3.6, 0.675, 0.238,
   3.7, 0.681, 0.240,
   3.8, 0.687, 0.242,
   3.9, 0.692, 0.244,
   4.0, 0.696, 0.245,
   4.2, 0.705, 0.246,
   4.4, 0.714, 0.245,
   4.6, 0.722, 0.243,
   4.8, 0.731, 0.240,
   5.0, 0.739, 0.236,
   5.5, 0.760, 0.223,
   6.0, 0.778, 0.209,
   6.5, 0.795, 0.196,
   7.0, 0.809, 0.185,
   7.5, 0.821, 0.174,
   8.0, 0.832, 0.165,
   9.0, 0.849, 0.148,
  10.0, 0.864, 0.135,
  11.0, 0.876, 0.123,
  12.0, 0.886, 0.113,
  14.0, 0.902, 0.098,
  16.0, 0.914, 0.086,
  18.0, 0.923, 0.077,
  20.0, 0.931, 0.069,
  25.0, 0.944, 0.056,
  30.0, 0.953, 0.047,
  35.0, 0.960, 0.040,
  40.0, 0.965, 0.035,
  50.0, 0.972, 0.028
};

static double
_f1(x) double x;
{
  double y;
  int i;

  if (x < 0.0) {
	fprintf(stderr, "_f1(x): out of range\n");
	y = 0.0;
  }
  else if (x == 0.0)
	y = 0.0;
  else if (x > 50.0)
	y = 1.0 - sqrt(2.0) / x;
  else {
	for (i = 1; x > tab1[i].x; i++)
		;
	if (x == tab1[i].x)
		y = tab1[i].f1;
	else
		y = tab1[i-1].f1
			+ (tab1[i].f1 - tab1[i-1].f1) * (x - tab1[i-1].x)
				/ (tab1[i].x - tab1[i-1].x);
  }
  return y;
}

static double
_f4(x) double x;
{
  double y;
  int i;

  if (x < 0.0) {
	fprintf(stderr, "_f1(x): out of range\n");
	y = 0.0;
  }
  else if (x == 0.0)
	y = 0.0;
  else if (x > 50.0)
	y = sqrt(2.0) / x;
  else {
	for (i = 1; x > tab1[i].x; i++)
		;
	if (x == tab1[i].x)
		y = tab1[i].f4;
	else
		y = tab1[i-1].f4
			+ (tab1[i].f4 - tab1[i-1].f4) * (x - tab1[i-1].x)
				/ (tab1[i].x - tab1[i-1].x);
  }
  return y;
}

static struct {
  double x, f2, f5;
} tab2[] = {
   0.0, 0.000, 0.000,
   0.5, 0.000, 0.000,
   1.0,-0.001,-0.002,
   1.2,-0.001,-0.003,
   1.4,-0.000,-0.005,
   1.6, 0.003,-0.006,
   1.8, 0.011,-0.007,
   2.0, 0.022,-0.010,
   2.2, 0.037,-0.015,
   2.4, 0.051,-0.022,
   2.6, 0.062,-0.028,
   2.8, 0.068,-0.033,
   3.0, 0.069,-0.034,
   3.2, 0.066,-0.033,
   3.4, 0.060,-0.030,
   3.6, 0.053,-0.026,
   3.8, 0.046,-0.021,
   4.0, 0.039,-0.017,
   4.2, 0.033,-0.014,
   4.4, 0.027,-0.011,
   4.6, 0.023,-0.009,
   4.8, 0.020,-0.008,
   5.0, 0.018,-0.007,
   5.5, 0.014,-0.006,
   6.0, 0.012,-0.006,
   7.0, 0.009,-0.006,
   8.0, 0.007,-0.005,
  10.0, 0.005,-0.004
};

static double
_f2(x) double x;
{
  double y, sqrt();
  int i;

  if (x < 0.0) {
	fprintf(stderr, "_f1(x): out of range\n");
	y = 0.0;
  }
  else if (x == 0.0)
	y = 0.0;
  else if (x > 10.0) {
	y = sqrt(2.0) / x;
	y = 0.25 * y * y;
  }
  else {
	for (i = 1; x > tab2[i].x; i++)
		;
	y = tab2[i-1].f2 + (tab2[i].f2 - tab2[i-1].f2) * (x - tab2[i-1].x)
		/ (tab2[i].x - tab2[i-1].x);
  }
  return y;
}

static double
_f5(x) double x;
{
  double y, sqrt();
  int i;

  if (x < 0.0) {
	fprintf(stderr, "_f1(x): out of range\n");
	y = 0.0;
  }
  else if (x == 0.0)
	y = 0.0;
  else if (x > 10.0) {
	y = sqrt(2.0) / x;
	y = y * y;
	y = 2.0 * y * y -0.25 * y;
  }
  else {
	for (i = 1; x > tab2[i].x; i++)
		;
	y = tab2[i-1].f5 + (tab2[i].f5 - tab2[i-1].f5) * (x - tab2[i-1].x)
		/ (tab2[i].x - tab2[i-1].x);
  }
  return y;
}


static struct {
  double x, f3, f6;
} tab3[] = {
   0.0, 0.09, 0.03,
   0.2, 0.09, 0.03,
   0.4, 0.09, 0.03,
   0.6, 0.08, 0.02,
   0.8, 0.08, 0.02,
   1.0, 0.06, 0.00,
   1.2, 0.02, 0.00,
   1.4, 0.00, 0.00,
   1.6, 0.02, 0.00,
   1.8, 0.05, 0.00,
   2.0, 0.07, 0.44,
   2.2, 0.08, 0.44,
   2.4, 0.08, 0.44,
   2.6, 0.10, 0.44,
   2.8, 0.12, 0.44,
   3.0, 0.15, 0.44,
   3.2, 0.19, 0.43,
   3.4, 0.24, 0.42,
   3.6, 0.29, 0.39,
   3.8, 0.35, 0.36,
   4.0, 0.41, 0.33,
   4.2, 0.46, 0.30,
   4.4, 0.51, 0.27,
   4.6, 0.56, 0.24,
   4.8, 0.60, 0.21,
   5.0, 0.64, 0.19,
   5.2, 0.67, 0.17,
   5.4, 0.69, 0.16,
   5.6, 0.70, 0.15,
   5.8, 0.71, 0.15,
   6.0, 0.72, 0.14,
   7.0, 0.76, 0.13,
   8.0, 0.79, 0.11,
   9.0, 0.83, 0.08,
  10.0, 0.85, 0.07,
  12.0, 0.87, 0.05,
  14.0, 0.89, 0.03,
  16.0, 0.90, 0.02,
  18.0, 0.93, 0.00,
  20.0, 0.93, 0.00
   };

static double
_f3(x) double x;
{
  double y;
  int i;

  if (x < 0.0) {
	fprintf(stderr, "_f1(x): out of range\n");
	y = 0.09;
  }
  else if (x == 0.0)
	y = 0.09;
  else if (x > 20.0)
	y = 1.0 - sqrt(2.0) / x;
  else {
	for (i = 1; x > tab3[i].x; i++)
		;
	y = tab3[i-1].f3 + (tab3[i].f3 - tab3[i-1].f3) * (x - tab3[i-1].x)
		/ (tab3[i].x - tab3[i-1].x);
  }
  return y;
}

static double
_f6(x) double x;
{
  double y;
  int i;

  if (x < 0.0) {
	fprintf(stderr, "_f1(x): out of range\n");
	y = 0.03;
  }
  else if (x == 0.0)
	y = 0.03;
  else if (x > 20.0)
	y = 0.0;
  else {
	for (i = 1; x > tab3[i].x; i++)
		;
	y = tab3[i-1].f6 + (tab3[i].f6 - tab3[i-1].f6) * (x - tab3[i-1].x)
		/ (tab3[i].x - tab3[i-1].x);
  }
  return y;
}
